/* bufpool.c - bpinit, bpget, bpfree */

#include <rtp.h>

struct bufpool bp;

/*------------------------------------------------------------------------
 *  bpinit - allocate memory for a buffer pool and link together
 *------------------------------------------------------------------------
 */
void
bpinit(void)
{
	char	*where;
	int	numbufs = RTP_BPBUFCNT;

	where = (char *) malloc(numbufs * RTP_BPBUFSZ);
	bp.bp_next = where;
	(void) sem_init(&bp.bp_sem, FALSE, numbufs);
	(void) pthread_mutex_init(&bp.bp_mutex, NULL);
	for (numbufs-- ; numbufs > 0 ; numbufs--, where += RTP_BPBUFSZ)
		*((char **) where) = where + RTP_BPBUFSZ;
	*((char **) where) = NULL;
}


/*------------------------------------------------------------------------
 * bpget - get a buffer from the pool
 *------------------------------------------------------------------------
 */
char *
bpget(void)
{
	char	*buf;
  
	(void) sem_wait(&bp.bp_sem);
	(void) pthread_mutex_lock(&bp.bp_mutex);
	buf = bp.bp_next;
	bp.bp_next = *((char **) buf);
	(void) pthread_mutex_unlock(&bp.bp_mutex);
	return buf;
}

/*------------------------------------------------------------------------
 * bpfree - free a buffer that was allocated from by getbuf
 *------------------------------------------------------------------------
 */
void
bpfree(void *buf)
{  
	(void) pthread_mutex_lock(&bp.bp_mutex);
	*((char **) buf) = bp.bp_next;
	bp.bp_next = (char *) buf;
	(void) pthread_mutex_unlock(&bp.bp_mutex);
	(void) sem_post(&bp.bp_sem);
}
